<?php
/**
 * (C) 2025 Motive Commerce Search Corp S.L. <info@motive.co>
 *
 * This file is part of Motive Commerce Search.
 *
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Motive (motive.co)
 * @copyright (C) 2025 Motive Commerce Search Corp S.L. <info@motive.co>
 * @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 */

namespace Motive\Prestashop\Builder;

use Motive\Prestashop\Model\Feed;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class FeedBuilder
 */
class FeedBuilder
{
    protected $context;

    /**
     * SchemaBuilder constructor.
     *
     * @param \Context $context
     */
    public function __construct(\Context $context)
    {
        $this->context = $context;
    }

    /**
     * Feed constructor.
     *
     * @param int $fromIdProduct
     * @param int $toIdProduct
     *
     * @return Feed
     */
    public function build($fromIdProduct = 0, $toIdProduct = -1)
    {
        return Feed::build(
            MetadataBuilder::build($this->context),
            $this->getProductBuilder($fromIdProduct, $toIdProduct)->get()
        );
    }

    /**
     * @param int $fromIdProduct
     * @param int $toIdProduct
     *
     * @return ProductBuilder
     *
     * @throws \Exception
     */
    protected function getProductBuilder($fromIdProduct, $toIdProduct)
    {
        return new ProductBuilder($this->context, $fromIdProduct, $toIdProduct);
    }
}
